function [sF,out_vars,variables,idToLocation,InitialConstant] = Location_SpaceExToStateflow(model,chart,base,doc,name,opt_cfg_reader,opt_display_flow,opt_display_invariant)
%
% ------------------------------------------------------------------------------
% author: Luan Viet Nguyen
% ------------------------------------------------------------------------------

    %position parmameter;
    x = 20;
    y = 120;
    w = 150;
    h = 150;
    % iterate over locations   
    position_x = 0;
    position_y = 0;
    for i_loc = 0 : base.getLocationCount() - 1
        loc = base.getLocation(i_loc);
        locations(i_loc + 1) = loc;
        % create corresponding location in SLSF model, 
        % need to be improved
        sF(i_loc+1) = Stateflow.State(chart);
        if (x+2*(position_x+1)*w < 2000)
            sF(i_loc+1).position = [x+2*(position_x+1)*w y+2*(position_y)*h w h] ; 
            position_x = position_x + 1;           
        else
            sF(i_loc+1).position = [x+2*(position_x+1)*w y+2*(position_y)*h  w h];
            position_x = 0;
            position_y = position_y + 1;
        end

        % create a map from location ids (from SpaceEx) to location objects (for creating transitions)
        idToLocation( loc.getId ) = locations(i_loc + 1); % TODO: error handling: what if ids not consecutive, etc.

        % print the flow
        if opt_display_flow
            ['Flow: ', expr_SpaceExToStateflow(loc.getFlow, EEqualityHandling.singleEq)]
        end
        % print the invariant
        if opt_display_invariant
            ['Invariant: ', expr_SpaceExToStateflow(loc.getInvariant, EEqualityHandling.doubleEq)]
        end  
        % return variable and flow description
        [variables, out_vars, flow, OutputDescription] = Variable_SpaceExToStateflow(expr_SpaceExToStateflow(loc.getFlow, EEqualityHandling.singleEq));
        % display flow equations, output in SLSF models
        FlowDescription = [char(locations(i_loc + 1).getName),10,'du:',10,flow,';',OutputDescription,10,'Location_out =',num2str(i_loc+1),';'];
        %FlowDescription = [char(locations(i_loc + 1).getName),10,'du:',10,flow,';',output];

        % add the flow into SLSF model
        sF(i_loc+1).LabelString = FlowDescription;

        % create a default transition
        % parse initial condition from javafiles
        if ~isempty(doc.getInitialStateConditions())
            initial = char(expr_SpaceExToStateflow(doc.getInitialStateConditions(), EEqualityHandling.doubleEq));
        else
        initial = '';
        end
        % get the inital value of variables and constants got from
        % configuration file
        [InitialVariable, InitialConstant] = Initial_SpaceExToStateflow(initial,variables,opt_cfg_reader);
        % set defaut transtion acquired from configuration field 
        if opt_cfg_reader
            if strfind(initial,locations(i_loc + 1).getName)
               defaultTransition = Stateflow.Transition(chart);
               defaultTransition.Destination = sF(i_loc + 1);
               dtA.DestinationOClock = 0;
               xsource = sF(i_loc + 1).Position(1)+sF(i_loc + 1).Position(3)/2;
               ysource = sF(i_loc + 1).Position(2)-30;
               dtA.SourceEndPoint = [xsource ysource];
               dtA.MidPoint = [xsource ysource+15];
               defaultTransition.LabelString = strcat('{',InitialVariable,'}');
            end
        end
    end
    
    % TODO: refactor next, it shouldn't be in a function called location translation...
    % add scope to see the outputs
    add_block('built-in/Scope',[name '/Scope'],'NumInputPorts',num2str(length(variables)+1),'Position',[350, 25, 400, 100],...
            'SaveToWorkspace', 'on', ...
        'SaveName', 'ScopeData', ...
        'DataFormat', 'Structure', ...
        'LimitDataPoints', 'off');
    
    %
    % create local, output, and input variables in SLSF model
    for i = 1: length(variables)
        sF_local = Stateflow.Data(chart);      
        sF_output = Stateflow.Data(chart); 
        sF_local.Name = variables{i};
        sF_local.Scope = 'Local';
        sF_local.Update = 'CONTINUOUS'; % Change update method of local variable to continuous
        sF_output.Name = out_vars{i};
        sF_output.Scope = 'Output';
        sF_output.Update = 'DISCRETE';	% Change update method of ouput to discrete
        % link the outputs to scope
        add_line(model.Name,[chart.Name,'/',num2str(i)],['Scope/',num2str(i)],'autorouting','on');     
    end
    % ouput location number
    sF_output = Stateflow.Data(chart); 
    sF_output.Name = 'Location_out';
    sF_output.Scope = 'Output';
    sF_output.Update = 'DISCRETE';
    add_line(model.Name,[chart.Name,'/',num2str(length(variables)+1)],['Scope/',num2str(length(variables)+1)],'autorouting','on');
end