function [iteration, generation_time] = ball_production(num_ball,num_loc,num_var)

%
% ------------------------------------------------------------------------------
% author: Luan Viet Nguyen
% ------------------------------------------------------------------------------

% randomly generate many examples of hybrid models whose behavior are
% similar to the bouncing ball system.
    javaaddpath(['.', filesep, 'lib', filesep, 'spaceex-converter.jar']);
    import de.uni_freiburg.informatik.swt.debugprinter.*;
    import de.uni_freiburg.informatik.swt.spaceexboogieprinter.*;
    import de.uni_freiburg.informatik.swt.spaceexxmlprinter.*;
    import de.uni_freiburg.informatik.swt.spaxeexxmlreader.*;
    import de.uni_freiburg.informatik.swt.sxhybridautomaton.*;
    import de.uni_freiburg.informatik.swt.sxlinearapproximation.*;
    % add classes and functions subfolders to path
    global symbolic_variable;
    addpath(genpath('functions'));
    addpath(genpath('classes'));
    
    %create symbolic variable and pass it to randgen function. 
    symbolic_variable = sym(zeros(1,num_var));
    for i = 1: num_var
        symbolic_variable(i)= sym(['x', num2str(i)], 'real');
    end 
    
    tstart = tic;
    old_dir = pwd();
    iteration = zeros(1,num_ball);
    generation_time = zeros(1,num_ball);

    for i = 1: num_ball
        [iteration(i), generation_time(i)] = batch_ball(num_loc,num_var);
        fclose all;
        cd(['..',filesep,'examples',filesep,'randgen_balls']);
        try
            % question? a .cfg file created by using java structure can not be closed or
            % changed its name, 
            % using copy to save different files instead
            java.io.File('randgen_ball.xml').renameTo(java.io.File( ['randgen_ball_',num2str(i),'.xml']));
            copyfile('randgen_ball.cfg',['randgen_ball_',num2str(i),'.cfg'])
            %java.io.File('randgen_ball.txt').renameTo(java.io.File( ['randgen_ball_',num2str(i),'.cfg']));                      
            %movefile('randgen_ball.xml', ['randgen_ball_',num2str(i),'.xml']);
            %movefile('randgen_ball.text', ['randgen_ball_',num2str(i),'.cfg']);
        catch    
        end
        cd(old_dir);
    end
    iter_average = mean(iteration);
    generation_time_average = mean(generation_time);

    % plot number of attempts to get a successful ball generation per trial
    figure;
    plot ((1:num_ball), iteration);
    hold on;
    plot ((1:num_ball), repmat(iter_average, 1, num_ball),'-.r');
    xlabel('Generation trial of bouncing ball example');
    ylabel('Generation attemps per trial');
    % plot similation times in second of differently individual trial
    figure;
    plot ((1:num_ball), generation_time);
    hold on;
    plot ((1:num_ball),repmat(generation_time_average, 1, num_ball),'-.r');
    xlabel('Generation trial of bouncing ball example');
    ylabel('Generation times per trial(seconds)');

    time = toc(tstart);
    fprintf('The execution time to generate %d examples of bouncing ball is %f seconds\n\n',num_ball,time);
    % display statistic table
    fprintf('The statistic table of %d bouncing ball model generations\n',num_ball);    
    Min = [min(iteration);min(generation_time)];
    Max = [max(iteration);max(generation_time)];
    Mean = [mean(iteration);mean(generation_time)];
    Median = [median(iteration);median(generation_time)];
    Variance = [var(iteration);var(generation_time)];
    Std = [std(iteration);std(generation_time)];
    Total = [sum(iteration);sum(generation_time)];
    Column = {'Iteration Number';'Generation Time(s)'};
    T = table(Min, Max, Mean, Median, Variance, Std, Total, 'RowNames', Column);
    disp(T);  
end