function [iteration, tElapsed] = batch_heater(num_loc,num_var)

%
% ------------------------------------------------------------------------------
% author: Luan Viet Nguyen
% ------------------------------------------------------------------------------

    % e.g run [iteration, generation_time] = batch_heater(2,1) to generate
    % the model whose behavior is similar to thermostat/heater example
    addpath(genpath('functions'));
    addpath(genpath('classes'));
    
    opt_debug = 1;
    iteration = 0;
    % sign= 1 if >=, = -1 if <= 
    sign = zeros(num_loc,num_var);
    tStart=tic;
    % initialize variables
    match_heater = 0; 
    dynamic = 0;
    while eig(dynamic)>=0 ||  match_heater == 0       
        %[dynamic, doc] = randgen_hybridautomaton(num_loc,num_var,'-s','-n','-ci');  
        [dynamic, doc] = randgen_hybridautomaton(num_loc,num_var,'-mg','-s','-rd','-ci'); 
        % get initial condition
        % check initial value should be equal or greater than inv_const_geq
        % and less than inv_const_leq
        initial = char(expr_SpaceExToStateflow(doc.getInitialStateConditions(), EEqualityHandling.doubleEq));
        if opt_debug
            ['Initial condition: ',initial];
        end    
        initial_split = strsplit(initial,'=');
        initial_value = str2double(initial_split{length(initial_split)});
        clear base;
        base = doc.getComponent(0);
        % reset variables
        match_flow_leq = 0;
        match_flow_geq = 0;
        % the smallest value of constant invariant (off state)
        inv_const_geq = 0;
        % the largest value of constant invariant (on state)
        inv_const_leq = 0; 
        % value of flow's constant in on state
        flow_leq_const = 0;      
        %%%%
        for i_loc = 0 : base.getLocationCount() - 1
            loc = base.getLocation(i_loc);
            invariant =  expr_SpaceExToStateflow(loc.getInvariant,EEqualityHandling.singleEq);
            flow =  expr_SpaceExToStateflow(loc.getFlow,EEqualityHandling.singleEq);
            if ~isempty(strfind(invariant,'<='))
                sign(i_loc+1) = -1;
                % split invariant to get variables and store their values
                inv_split = strsplit(invariant,'<=');
                inv_const_leq = str2double(inv_split{length(inv_split)});
                % check flow in a correct form of On State e.g;x1=6-.07923*x1; x1=5.2-3.07923*x1
                flow_leq = regexp(flow, '\w*=(?=.*[1-9])(?:[1-9]\d*\.?|0?\.)\d*-(\d*)(.)?\d+\*\w+', 'match');
                if ~isempty(flow_leq);
                    match_flow_leq = 1;
                    % the flow should have a constant which is greater than
                    % invariant constant
                    flow_tmp = regexp(char(flow_leq),'=(?=.*[1-9])(?:[1-9]\d*\.?|0?\.)\d*', 'match');
                    flow_leq_const = str2double(char(regexprep(char(flow_tmp),'=','')));
                end
            else
                sign(i_loc+1) = 1;
                % split invariant to get variables and store their values
                inv_split = strsplit(invariant,'>=');
                inv_const_geq = str2double(inv_split{length(inv_split)}); 
                % check flow in a correct form of Off State e.g x1=0-.07923*x1 
                if ~isempty(regexp(flow, '\w*=(0)?-(\d*)(.)?\d+\*\w+', 'match'))
                    match_flow_geq = 1;
                end
            end
            if opt_debug
                ['Flow: ', flow]
                ['Invariant: ', invariant]
            end
        end 
        if inv_const_geq < initial_value && initial_value < inv_const_leq && inv_const_leq ...
                < flow_leq_const && sum(sign) == 0 && match_flow_leq == 1 && match_flow_geq == 1
           match_heater = 1; 
        else
            match_heater = 0;
        end
        iteration = iteration +1;
    end
    % generation time
    tElapsed=toc(tStart);
      
end
    