classdef AuxVarMap
%AUXVARMAP A map from predefined names to fresh names in the Stateflow model.
% Manual: to insert a new variable, add to the following places:
% 1) properties; 2) elemCell
%
% ------------------------------------------------------------------------------
% author: Christian Schilling
% ------------------------------------------------------------------------------

    properties (Constant = true)
        PREFIX_ASSIGNMENT = 't_';
        PREFIX_VARIABLE = 'v_';
        PREFIX_OUTPUT = 'o_';
        
        % stopping signal
        stop = 'a_stop';
        % current time
        t = 'a_t';
        % backtracking information for reading correct data
        backtrack = 'a_backtrack';
        % min. dwelling time
        T = 'a_T';
        % time the invariant was first violated
        tFail = 'a_tFail';
        % temporary time (for backtracking)
        tTmp = 'a_tTmp';
        % output time
        tOut = 'a_tOut';
        % maximum simulation time
        maxT = 'a_MAX_T';
        % transition array
        transArray = 'a_transArray';
        % current transition array index
        transIdx = 'a_transIdx';
        % current transition
        trans = 'a_trans';
        % current location
        loc = 'a_loc';
        % current number of resets
        resets = 'a_resets';
        % maximum number of resets
        maxResets = 'a_RESETS';
        % random number
        rand = 'a_rand';
        % random array
        randArray = 'a_randA';
        % initial state index
        init = 'a_init';
        % epsilon for equality handling
        eps = 'a_EPS';
        % random seed
        seed = 'a_SEED';
    end
    
    methods (Access = public, Static = true)
        % encoding:
        % 1) name,
        % 2) variable scope [l(ocal), o(utput), p(arameter), i(nput)],
        % 3) update type [d(iscrete), c(ontinuous)],
        % 4) data type [i(nteger), d(ouble), a(rray), type name]
        function elems = elemCell()
            elems = {...
                % must be the first element
                AuxVarMap.stop, 'o', 'd', 'boolean'; ...
                % must be the second element
                AuxVarMap.tOut, 'o', 'd', 'd'; ...
                % must be the third element
                AuxVarMap.backtrack, 'o', 'd', 'i'; ...
                % any order
                AuxVarMap.T, 'l', 'd', 'd'; ...
                AuxVarMap.t, 'l', 'c', 'd'; ...
                AuxVarMap.tFail, 'l', 'd', 'd'; ...
                AuxVarMap.tTmp, 'l', 'd', 'd'; ...
                AuxVarMap.maxT, 'p', 'd', 'd'; ...
                AuxVarMap.transArray, 'l', 'd', 'a'; ...
                AuxVarMap.transIdx, 'l', 'd', 'i'; ...
                AuxVarMap.trans, 'l', 'd', 'i'; ...
                AuxVarMap.loc, 'o', 'd', 'i'; ...
                AuxVarMap.resets, 'l', 'd', 'i'; ...
                AuxVarMap.maxResets, 'p', 'd', 'i'; ...
                AuxVarMap.rand, 'i', 'c', 'd'; ...
                AuxVarMap.randArray, 'i', 'd', 'a'; ...
                AuxVarMap.init, 'l', 'd', 'i'; ...
                AuxVarMap.eps, 'p', 'd', 'd'; ...
                AuxVarMap.seed, 'p', 'd', 'i' ...
            };
        end
    end
end