function [dynamic, rand_cons_output] = dynamics_randgen_hybridautomaton(num_loc,num_var,opt_zero, opt_negative,opt_positive_negative,...
                                       opt_negative_zero,opt_postive_negative_zero, opt_positive_real,opt_negative_real,opt_imaginary,...
                                       rand_cons_invariant_pos,rand_cons_invariant_neg)                                   
%
% ------------------------------------------------------------------------------
% author: Luan Viet Nguyen
% ------------------------------------------------------------------------------                                   
    % randomly select different dynamics
    rand_cons_output = [];
    % Flow dynamics
    % random_matrix = rand(num_var);
    % create flow dynamics for each location, all eigen values are
    % positive by a default
    random_matrix = rand(num_var);            
    % create a stable matrix with zero eigen values 
    random_matrix_zero = triu(rand(num_var),1);
    % display_eigenvalues = eig(random_matrix_zero);
    % create a stable matrix with all negative eigen values 
    eigenvalues_neg = diag(-abs(randn(num_var,1))); % Random negative eigenvalues
    unitary_matrix = orth(randn(num_var)); % Random unitary matrix               
    random_matrix_neg = unitary_matrix*eigenvalues_neg*unitary_matrix'; % Matrix stable_matrix has the above negative eigenvalues
    % create a stable matrix with combination of negative and zero eigen values 
    eigenvalues_neg_zero = eigenvalues_neg;
    for i = 1: randi([1,num_var])
        eigenvalues_neg_zero(:,randi([1,num_var])) = 0;
    end
    random_matrix_neg_zero = unitary_matrix*eigenvalues_neg_zero*unitary_matrix';

    % create a stable matrix with combination of positive and negative eigen values 
    eigenvalues_pos_neg = eigenvalues_neg;
    for i = 1: randi([1,num_var])
        eigenvalues_pos_neg(:,randi([1,num_var])) = -eigenvalues_pos_neg(:,randi([1,num_var]));
    end
    random_matrix_pos_neg = unitary_matrix*eigenvalues_pos_neg*unitary_matrix';
    %display_eigenvalues = eig(random_matrix_pos_neg);
    % Create a matrix with postive real part eigen values 
    random_matrix_pos_real = rand(num_var);
    for i = 1: length(random_matrix_pos_real)
        for j = 1: length(random_matrix_pos_real)
            if j > i
                random_matrix_pos_real(i,j) = - random_matrix_pos_real(i,j);
            end
        end    
    end 
    % Create a stable matrix with negataive real part eigen values
    random_matrix_neg_real = -random_matrix_pos_real;
    % Create a stable matrix with only imaginary eigen values 
    random_matrix_skew = random_matrix - random_matrix';
    % Create a matrix randomly produced a combination of positive negative and zero eigenvalues or just only one type of eigenvalues 
    eigenvalues_pos_neg_zero  = diag(rand(num_var)*randi([-1,1],num_var,1));
    random_matrix_pos_neg_zero = unitary_matrix* eigenvalues_pos_neg_zero*unitary_matrix';
    %display_eigenvalues = eig(random_matrix_pos_neg_zero)
    rand_cons_output = rand_cons_invariant_pos;
    if opt_zero == 1;
        random_matrix = random_matrix_zero; % Matrix stable_matrix has all zero eigenvalues
        %rand_cons_zero = randi([-1,1],1,1)*random_matrix_neg_real + randi([-1,1],1,1)*random_matrix_pos_real;
        random_cons_output_tmp = rand(num_loc,num_var);
        rand_cons_output = randi([-1,1],1,1)* random_cons_output_tmp + randi([-1,1],1,1)* (-random_cons_output_tmp);
    elseif opt_negative == 1;
        random_matrix = random_matrix_neg; % Matrix stable_matrix has the above negative eigenvalues
        %inv_op = leq_op;
        %guard_op = geq_op;
        %rand_cons_output = rand_cons_invariant_neg;
        rand_cons_output = rand_cons_invariant_pos;
    elseif opt_negative_zero == 1;
        random_matrix = random_matrix_neg_zero;
        %inv_op = leq_op;
        %guard_op = geq_op;
        rand_cons_output = rand_cons_invariant_neg;
    % Matrix can be stable or unstable that depend on the random combination of positive and negative eigen values    
    elseif opt_positive_negative == 1;
        random_matrix = random_matrix_pos_neg;    
        rand_cons_output = rand_cons_invariant_pos;
    % Matrix can be stable or unstable that depend on the random combination of positive, negative and zero eigen values    
    elseif opt_postive_negative_zero == 1;
        random_matrix = random_matrix_pos_neg_zero;    
        rand_cons_output = rand_cons_invariant_pos;
    % Use skew-symmetric matrix
    elseif opt_imaginary == 1;
        random_matrix = random_matrix_skew;
        rand_cons_output = rand_cons_invariant_pos;
    % Use radom matrix produced postive real part complex eigenvalues    
    elseif opt_positive_real == 1;
        random_matrix = random_matrix_pos_real;
        rand_cons_output = rand_cons_invariant_pos;
    % Use radom matrix produced negative real part complex eigenvalues    
    elseif opt_negative_real == 1;
        random_matrix = random_matrix_neg_real;
        rand_cons_output = rand_cons_invariant_pos;
    end
    % output the dynamics
    dynamic = random_matrix;
end

% function dispDebug(msg)
%     opt_debug = 1;
%     if opt_debug
%         disp(msg);
%     end
% end
