%%%%%%%%%% Random Hybrid Automaton (RandHA) Generation %%%%%%%%%%
% Execution options:
%% call randgen_hybridautomaton('-g') to generate RandHA included a self-loop guard implementation 
%% call randgen_hybridautomaton('-t') to generate RandHA with a dwell-time guard implementation
%% call randgen_hybridautomaton('-s') to generate RandHA with a global clock
%% call randgen_hybridautomaton('-n') to generate RandHA with a stable dynamics( negative eigenvalues )
%% call randgen_hybridautomaton('-i') to generate RandHA with a stable dynamics( only imaginary eigenvalues )
%% call randgen_hybridautomaton('-pn') to generate RandHA with a stable dynamics( positive and negative eigenvalues)
%% call randgen_hybridautomaton('-pnz') to generate RandHA that has stable or unstable dynamics( combination of positive, negative and zero eigenvalues)
%% call randgen_hybridautomaton('-nr') to generate RandHA with a stable dynamics( negative real part eigenvalues)
%% call randgen_hybridautomaton('-pr') to generate RandHA with a stable dynamics( postive real part eigenvalues)
%% note: the default unstable dynamics has positve eigenvalues
%% call randgen_hybridautomaton('-ci') to create a constant invariant
%% call randgen_hybridautomaton('-si') to create a symbolic invariant
%% call randgen_hybridautomaton('-cr') to generate RandHA with a constant reset
%% call randgen_hybridautomaton('-sr') to generate RandHA with a reset to a fucntion of variables
%% call randgen_hybridautomaton('-rd') to generate RandHA with a random dynamics
%% call randgen_hybridautomaton('-mg') to save multiple examples generation in specific folder
%% e.g randgen_hybridautomaton('-s','-n','-si','-r','-t') to generate hybrid automation included global clock, negative eigenvalues matrix,
%% symbolic invariant, reset variable and dwell-time guard implementation 

% ------------------------------------------------------------------------------
% author: Luan Viet Nguyen
% ------------------------------------------------------------------------------

function [out] = randgen_hybridautomaton(num_loc,num_var,varargin)
    clear base locations Variable_Params Variable_Params_Net Variable_Map;
    global symbolic_variable;
    % save symbolic variable, it's some how clear when
    symbolic_variable_local = symbolic_variable;
    
    % Specify options from the input argument
    [opt_guard_selfloop, opt_dwelltime,  opt_sanity_checking,opt_spaceExtoStateflow, opt_zero, opt_negative, opt_positive_negative,opt_negative_zero,...
    opt_postive_negative_zero, opt_positive_real,opt_negative_real,opt_imaginary,opt_random_dynamics,opt_constant_invariant,...
    opt_symbolic_invariant, opt_constant_reset,  opt_symbolic_reset, opt_multi_generation] = option_randgen_hybridautomaton(varargin);
    if opt_multi_generation == 0;
        javaaddpath(['.', filesep, 'lib', filesep, 'spaceex-converter.jar']);
        import de.uni_freiburg.informatik.swt.debugprinter.*;
        import de.uni_freiburg.informatik.swt.spaceexboogieprinter.*;
        import de.uni_freiburg.informatik.swt.spaceexxmlprinter.*;
        import de.uni_freiburg.informatik.swt.spaxeexxmlreader.*;
        import de.uni_freiburg.informatik.swt.sxhybridautomaton.*;
        import de.uni_freiburg.informatik.swt.sxlinearapproximation.*;
    end
    doc = de.uni_freiburg.informatik.swt.sxhybridautomaton.SpaceExDocument;
    base = de.uni_freiburg.informatik.swt.sxhybridautomaton.BaseComponent(doc);    
    % Creating one components, network component to do a reachability analysis in SpaceEx
    base.setID('rand_gen');
    %base.getID;
    %comp = de.uni_freiburg.informatik.swt.sxhybridautomaton.Component(doc);
    %comp.setID(rand_gen);
    %base.getLocationCount
    % return SpaceEx XML Document
    out = doc;
    % # parameters:
    % # min_locations
    % # max_locations
    % 
    %num_loc_min = 1;
    %num_loc_max = loc_max;
    %num_loc = randi([num_loc_min, num_loc_max],1,1);
    % 
    % # parameters:
    % # min_variables
    % # max_variables
    % disp('aaaaaaaaa');
    %num_var_min = 2;
    %num_var_max = var_max;
    %num_var = randi([num_var_min, num_var_max],1,1)
    %b = symbolic_variable    
    if opt_multi_generation
        x_sys = symbolic_variable_local;
    else
        for i = 1: num_var
            x_sys(i)= sym(['x', num2str(i)], 'real');
        end  
    end
    x_sys_clock = x_sys;
    
    % return adjacency matrix 
    [adjacency_matrix] = adj_matrix_randgen_hybridautomaton(opt_guard_selfloop,num_loc);
    % create different random constant matrixs
    inv_min = 1;
    inv_max = 10;
    rand_cons_invariant_pos = randi([inv_min,inv_max],num_loc,num_var);
    rand_cons_invariant_neg = -rand_cons_invariant_pos;
    % add random symbolic invariant
    rand_syms_invariant = sym(zeros(num_loc,num_var));
    for i_loc = 1: num_loc   
        for i_var = 1: num_var
            rand_syms_invariant(i_loc,i_var) = sym(char(x_sys*randi([-1,1],num_var,1)));
        end
    end
    % set invariant for t
    t_min = 3;
    %t_max = 3*num_loc;
    t_max = 10;
    T = randi([t_min, t_max],num_loc,1);
    %gplotdc(adjacency_matrix,rand(num_loc,2)); 
    % Set ID for each location
    for i_loc = 1 : num_loc % or whatever representation of number of locations you use
        loc = de.uni_freiburg.informatik.swt.sxhybridautomaton.Location(base);
        locations(i_loc) = loc;
        loc.setId(i_loc);
        loc.setName(['location_',num2str(i_loc)]);
        % return random dynamics matrix and random constant matrix of
        % invariant
        if opt_random_dynamics == 1
            mode = randi([0,8],1,1);
            switch mode
                case 0
                    opt_zero = 1;
                    dispDebug('Dynamics has all zero eigen values'); 
                case 1
                    opt_negative = 1;
                    dispDebug('Dynamics has all negative eigen values'); 
                case 2
                    opt_positive_negative = 1;
                    dispDebug('Dynamics has a combination of positive and negative eigen values');
                case 3
                    opt_negative_zero = 1;
                    dispDebug('Dynamics has a combination of zero and negative eigen values');
                case 4
                    opt_postive_negative_zero = 1;
                    dispDebug('Dynamics has a combination of positive, zero and negative eigen values');
                case 5
                    opt_positive_real = 1;
                    dispDebug('Dynamics has a positive real-part of complex eigen values');
                case 6
                    opt_negative_real= 1;
                    dispDebug('Dynamics has a positive negative-part of complex eigen values');
                case 7    
                    opt_imaginary = 1;
                    dispDebug('Dynamics has a purely imaginary eigen values');
                case 8    
                    dispDebug('Default dynamics with all positive eigen values'); 
            end 
        end  
        [random_matrix, rand_cons_invariant] = dynamics_randgen_hybridautomaton(num_loc, num_var, opt_zero, opt_negative,opt_positive_negative,...
                                                opt_negative_zero,opt_postive_negative_zero, opt_positive_real,opt_negative_real,opt_imaginary,...
                                                                                                rand_cons_invariant_pos,rand_cons_invariant_neg);   
        % return dynamic
        dynamic = random_matrix;
        %dynamic = random_matrix;
        [flow] = flow_randgen_hybridautomaton(num_var,x_sys,random_matrix,opt_zero,opt_dwelltime);
        %Set flow
        flow_parse = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLFormulaParser(regexprep(flow,'''=','=='));
        expr = flow_parse.get;
        locations(i_loc).setFlow(expr);
        % choose different invariant set for each type of flow dynamics
        % invariant operation
        % inv_op = '';
        % guard_op = '';
        leq_op = '<=';
        geq_op = '>=';
        mode = randi([0,1],1,1);
        switch mode
            case 0
                inv_op = leq_op;
                guard_op = geq_op;
            case 1   
                inv_op = geq_op;
                guard_op = leq_op;
        end    
        % randomly generate invariant for each location
        [invariant] = invariant_randgen_hybridautomaton(x_sys,i_loc,num_var,inv_op, opt_dwelltime, opt_constant_invariant,...
                            opt_symbolic_invariant, opt_imaginary, opt_zero, rand_cons_invariant, rand_syms_invariant,T);  
        invariant_parse = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLFormulaParser(invariant);
        expr_invariant = invariant_parse.get;
        locations(i_loc).setInvariant(expr_invariant);
        trans_count = zeros(1,num_loc);
        for j_loc = 1 : num_loc
            if adjacency_matrix(i_loc, j_loc) == 1 % transition from location i to j
                trans = de.uni_freiburg.informatik.swt.sxhybridautomaton.Transition(base);
                trans.setSource(i_loc); %set source location to i
                trans.setTarget(j_loc);  %set destination j
                trans_count(i_loc) = trans_count(i_loc)+ 1;
                % set a guard for each transition
                [guard] = guard_randgen_hybridautomaton(x_sys,i_loc,num_var, trans_count,guard_op, opt_dwelltime,opt_constant_invariant,...
                            opt_symbolic_invariant, opt_imaginary, opt_zero, rand_cons_invariant, rand_syms_invariant,T); 
                guard_parse = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLFormulaParser(guard);
                expr_guard = guard_parse.get;
                trans.setGuard(expr_guard);
                expr_SpaceExToStateflow(trans.getGuard,false);
                %expr_SpaceExToStateflow(trans.getGuard,EEqualityHandling.doubleEq)
                % set an assigment for each transition
                [reset] = reset_randgen_hybridautomaton(x_sys,i_loc,num_var, opt_dwelltime,...
                        opt_zero,opt_constant_reset, opt_symbolic_reset,rand_cons_invariant);  
                reset_parse = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLFormulaParser(reset);
                expr_reset= reset_parse.get;
                trans.setAssignment(expr_reset);
                %expr_SpaceExToStateflow(trans.getAssignment,EEqualityHandling.doubleEq)
            end
        end    
    end
    % Add a random initial condition for model without adding clock
    initialStateConditions = ['loc(',char(base.getID),'_1)=location_', num2str(randi([1,num_loc],1,1))];
    for i = 1 : num_var % or whatever representation of number of locations you use
        initialStateConditions = [initialStateConditions,'& ','x', num2str(i),'=',num2str(randi([1, 30],1,1))];
    end
    if opt_dwelltime == 1
        initialStateConditions = [initialStateConditions,'& ','t=0'];
    end
    %initialStateConditions
    initialcondition_parse = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLFormulaParser(initialStateConditions);
    expr_initial = initialcondition_parse.get;
   
    doc.setInitialStateConditions(expr_initial);
    % expr_SpaceExToStateflow(doc.getInitialStateConditions,EEqualityHandling.doubleEq);
    % Building network component
    Net = de.uni_freiburg.informatik.swt.sxhybridautomaton.NetworkComponent(doc);
    Net.setID('sys_rand_gen');
    % Net.getID
    % Set bind
    bind = de.uni_freiburg.informatik.swt.sxhybridautomaton.Bind(Net, char(base.getID),[char(base.getID),'_1'], 295, 170, 58, 62);
    % Set parameter type and dynamics
    Param_real = de.uni_freiburg.informatik.swt.sxhybridautomaton.ParamType.REAL;
    Param_dynamics = de.uni_freiburg.informatik.swt.sxhybridautomaton.ParamDynamics.ANY;
    for i = 1 : num_var
        % Variable of basecomponent
        Variable_Params = de.uni_freiburg.informatik.swt.sxhybridautomaton.VariableParam(base,char(x_sys(i)),Param_real,Param_dynamics,false);
        % Variable of Network component
        Variable_Params_Net = de.uni_freiburg.informatik.swt.sxhybridautomaton.VariableParam(Net,char(x_sys(i)),Param_real,Param_dynamics,false);
        % Map the variable de.uni_freiburg.informatik.swt.sxhybridautomaton.VariableParam(Net,char(x_sys(i)),Param_real,Param_dynamics,false);
        Variable_Map = de.uni_freiburg.informatik.swt.sxhybridautomaton.ParamMap(bind, char(x_sys(i)),char(x_sys(i)));
        % adding the output variables, out format, etc., into configuration file
        doc.addOutputVar(char(x_sys(i)));
        doc.setOutputFormat('GEN');
        % 
        %doc.setTimeHorizon(5);
        %doc.setMaxIterations(20);
    end
    if opt_dwelltime == 1
        % add variable t
        de.uni_freiburg.informatik.swt.sxhybridautomaton.VariableParam(base,'t',Param_real,Param_dynamics,false);
        de.uni_freiburg.informatik.swt.sxhybridautomaton.VariableParam(Net,'t',Param_real,Param_dynamics,false);
        de.uni_freiburg.informatik.swt.sxhybridautomaton.ParamMap(bind, 't', 't');
        doc.addOutputVar('t');
    end
    % Set the systemID
    doc.setSystemID(Net.getID)
    
    % Save the SpaceEx document and the configuration file without adding
    xml_printer = de.uni_freiburg.informatik.swt.spaceexxmlprinter.SpaceExXMLPrinter(doc);
    % cd(['..', filesep, 'examples']);
    old_dir = pwd();
    cd(['..', filesep, 'examples', filesep, 'randgen' ]);
    xml_printer.saveCFG(['randgen_',num2str(num_loc),'locs_',num2str(num_var),'vars.cfg']);
    cd(old_dir)
    %cd(['..', filesep, 'spaceex2stateflow']);
    xml_printer.saveXML(['randgen_',num2str(num_loc),'locs_',num2str(num_var),'vars.xml']);
    try
    	movefile(['randgen_',num2str(num_loc),'locs_',num2str(num_var),'vars.xml'],['..', filesep, 'examples', filesep, 'randgen']);
    catch
    end
    %SpaceExToStateflow('randgen.xml','-f');
    %SpaceExToStateflow('randgen.xml','randgen.cfg');
    % Adding the global time clock for sanity checking and save it as two
    % differents xml and cfg files
    if opt_sanity_checking == 1
        doc_clock = de.uni_freiburg.informatik.swt.sxhybridautomaton.SpaceExDocument;
        doc_clock.addComponent(base);
        base_clock = de.uni_freiburg.informatik.swt.sxhybridautomaton.BaseComponent(doc_clock);
        base_clock.setID('clock');
        base_clock.getID;
        loc_clock = de.uni_freiburg.informatik.swt.sxhybridautomaton.Location(base_clock);
        loc_clock.setName('clock');
        flow_clock = 'time''= 1'; 
        flow_parse_clock = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLFormulaParser(regexprep(flow_clock,'''=','=='));
        expr_clock = flow_parse_clock.get;
        loc_clock.setFlow(expr_clock);
        invariant_time = '0 <= time <= tmax';
        invariant_parse_time = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLFormulaParser(invariant_time);
        expr_invariant_time = invariant_parse_time.get;
        loc_clock.setInvariant(expr_invariant_time);     
        % add time max
        Param_real_tmax = de.uni_freiburg.informatik.swt.sxhybridautomaton.ParamType.REAL;
        Param_dynamics_tmax = de.uni_freiburg.informatik.swt.sxhybridautomaton.ParamDynamics.CONST;
        % Assign network component
        doc_clock.addComponent(Net);
        bind_clock = de.uni_freiburg.informatik.swt.sxhybridautomaton.Bind(Net, char(base_clock.getID),[char(base_clock.getID),'_1'], 523, 393, 58, 62);
        Variable_Time = de.uni_freiburg.informatik.swt.sxhybridautomaton.VariableParam(base_clock,'time',Param_real,Param_dynamics,false);
        Variable_Time_Net = de.uni_freiburg.informatik.swt.sxhybridautomaton.VariableParam(Net,'time',Param_real,Param_dynamics,false);
        Variable_Time_Map = de.uni_freiburg.informatik.swt.sxhybridautomaton.ParamMap(bind_clock, 'time','time');
        % adding constant tmax
        Variable_tmax = de.uni_freiburg.informatik.swt.sxhybridautomaton.VariableParam(base_clock,'tmax',Param_real_tmax,Param_dynamics_tmax,false);
        Variable_tmax_Net = de.uni_freiburg.informatik.swt.sxhybridautomaton.VariableParam(Net,'tmax',Param_real_tmax,Param_dynamics_tmax,false);
        Variable_tmax_Map = de.uni_freiburg.informatik.swt.sxhybridautomaton.ParamMap(bind_clock, 'tmax','tmax');
        xml_printer_clock = de.uni_freiburg.informatik.swt.spaceexxmlprinter.SpaceExXMLPrinter(doc_clock);
        % Save the SpaceEx document and the configuration file with adding
        % Add a random initial condition for model with adding clock
        %initialStateConditions_clock =  [initialStateConditions,'& time=0'];
        initialStateConditions_clock =  [initialStateConditions,'& time=0& tmax=100'];
        initialcondition__clock_parse = de.uni_freiburg.informatik.swt.spaxeexxmlreader.SpaceExXMLFormulaParser(initialStateConditions_clock);
        expr__clock_initial = initialcondition__clock_parse.get; 
        doc_clock.setInitialStateConditions(expr__clock_initial);
        % adding the output variables
        for i = 1 : num_var
            doc_clock.addOutputVar(char(x_sys_clock(i)));
        end
        doc_clock.addOutputVar('time');
        if opt_dwelltime == 1
            doc_clock.addOutputVar('t');
        end
        doc_clock.setOutputFormat('GEN');
        % 
        % doc.setTimeHorizon(5);
        % doc.setMaxIterations(20);  
        % Set the systemID
        doc_clock.setSystemID(Net.getID)
        old_dir = pwd();
        if opt_multi_generation == 1
            % specify which folder you want to save bouncing balls or heater
            % examples
            cd(['..',filesep,'examples',filesep,'randgen_balls']);
            %cd(['..',filesep,'examples',filesep,'randgen_heaters']);
            xml_printer_clock.saveCFG(['randgen_ball','.cfg']);
            %xml_printer_clock.saveCFG(['randgen_heater','.cfg']);
            fclose('all');
            cd(old_dir);
            xml_printer_clock.saveXML(['randgen_ball','.xml']);
            %xml_printer_clock.saveXML(['randgen_heater','.xml']);
        else
            cd(['..',filesep,'examples',filesep,'randgen']);
            xml_printer_clock.saveCFG(['randgen_clock_',num2str(num_loc),'locs_',num2str(num_var),'vars.cfg']);
            cd(old_dir);
            %cd(['..', filesep, 'spaceex2stateflow']);
            xml_printer_clock.saveXML(['randgen_clock_',num2str(num_loc),'locs_',num2str(num_var),'vars.xml']);
        end    
        
        try
            if opt_multi_generation 
                movefile(['randgen_ball','.xml'],['..', filesep, 'examples', filesep, 'randgen_balls']);
                %movefile(['randgen_heater','.xml'],['..', filesep, 'examples', filesep, 'randgen_heaters']);
            else
                movefile(['randgen_clock_',num2str(num_loc),'locs_',num2str(num_var),'vars.xml'],['..', filesep, 'examples', filesep, 'randgen']);
            end
        catch
        end
        %cd(old_dir);
    end
    if opt_spaceExtoStateflow == 1
        xml_filename = ['randgen_',num2str(num_loc),'locs_',num2str(num_var),'vars.xml'];
        cfg_filename = ['randgen_',num2str(num_loc),'locs_',num2str(num_var),'vars.cfg'];
        SpaceExToStateflow(xml_filename, cfg_filename,'-r');
    %else
    %SpaceExToStateflow(['randgen_',num2str(num_loc),'locs_',num2str(num_var),'vars.xml'],['randgen_',num2str(num_loc),'locs_',num2str(num_var),'vars.cfg'],'-r');
    end
    %SpaceExToStateflow(strcat('randgen_','locs_','vars.xml'),strcat('randgen_','locs_','vars.cfg'),'-r');
end

function dispDebug(msg)
    opt_debug = 0;
    if opt_debug
        disp(msg);
    end
end